@@ -39,8 +39,11 @@
 #'   \item{\strong{"SD2"}: }{The circular successive-differences variance estimator described by Ash (2014).
 #'   This estimator is the basis of the "successive-differences replication" estimator commonly used
 #'   for variance estimation for systematic sampling.}
+#'   \item{\strong{"Deville-Tille"}: }{The estimator of Deville and Tillé (2005),
+#'   developed for balanced sampling using the cube method.
+#'   }
 #' }
-#' @param probs Required if \code{variance_estimator} equals \code{"Deville-1"} or \code{"Deville-2"}.
+#' @param probs Required if \code{variance_estimator} equals \code{"Deville-1"}, \code{"Deville-2"}, or \code{"Breidt-Chauvet"}.
 #' This should be a matrix or data frame of sampling probabilities.
 #' If there are multiple stages of sampling,
 #' then \code{probs} can have multiple columns,
@@ -65,22 +68,25 @@
 #' with one column for each level of sampling to be accounted for by the variance estimator.
 #' @param sort_order Required if \code{variance_estimator} equals \code{"SD1"} or \code{"SD2"}.
 #' This should be a vector that orders the rows of data into the order used for sampling.
+#' @param aux_vars Required if \code{variance_estimator} equals \code{"Deville-Tille"}.
+#' A matrix of auxiliary variables.
 #' @section Variance Estimators:
 #' See \link[svrep]{variance-estimators} for a
 #' description of each variance estimator.
 #' @section Arguments required for each variance estimator:
 #' Below are the arguments that are required or optional for each variance estimator.
 #'
-#' | variance_estimator       | probs      | joint_probs | cluster_ids | strata_ids  | strata_pop_sizes | sort_order |
-#' | ------------------------ |-----------:| -----------:| -----------:| -----------:| ----------------:|-----------:|
-#' | Stratified Multistage SRS|            |             | Required    | Required    | Required         |            |
-#' | Ultimate Cluster         |            |             | Required    | Required    | Optional         |            |
-#' | SD1                      |            |             | Required    | Optional    | Optional         | Required   |
-#' | SD2                      |            |             | Required    | Optional    | Optional         | Required   |
-#' | Deville-1                | Required   |             | Required    | Optional    |                  |            |
-#' | Deville-2                | Required   |             | Required    | Optional    |                  |            |
-#' | Yates-Grundy             |            | Required    |             |             |                  |            |
-#' | Horvitz-Thompson         |            | Required    |             |             |                  |            |
+#' | variance_estimator       | probs      | joint_probs | cluster_ids | strata_ids  | strata_pop_sizes | sort_order | aux_vars |
+#' | ------------------------ |-----------:| -----------:| -----------:| -----------:| ----------------:|-----------:|---------:|
+#' | Stratified Multistage SRS|            |             | Required    | Required    | Required         |            |          |
+#' | Ultimate Cluster         |            |             | Required    | Required    | Optional         |            |          |
+#' | SD1                      |            |             | Required    | Optional    | Optional         | Required   |          |
+#' | SD2                      |            |             | Required    | Optional    | Optional         | Required   |          |
+#' | Deville-1                | Required   |             | Required    | Optional    |                  |            |          |
+#' | Deville-2                | Required   |             | Required    | Optional    |                  |            |          |
+#' | Deville-Tille            | Required   |             | Required    | Optional    |                  |            | Required |
+#' | Yates-Grundy             |            | Required    |             |             |                  |            |          |
+#' | Horvitz-Thompson         |            | Required    |             |             |                  |            |          |
 #' @md
 #' @return The matrix of the quadratic form representing the variance estimator.
 #' @export
@@ -167,12 +173,13 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
                                   cluster_ids = NULL,
                                   strata_ids = NULL,
                                   strata_pop_sizes = NULL,
-                                  sort_order = NULL) {
+                                  sort_order = NULL,
+                                  aux_vars = NULL) {
 
   accepted_variance_estimators <- c(
     "Yates-Grundy", "Horvitz-Thompson",
     "Ultimate Cluster", "Stratified Multistage SRS",
-    "SD1", "SD2", "Deville-1", "Deville-2"
+    "SD1", "SD2", "Deville-1", "Deville-2", "Deville-Tille"
   )
 
   if (length(variance_estimator) > 1) {
@@ -199,12 +206,12 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
 
   # Check inputs and assemble all necessary information
   # for estimators of stratified/clustered designs
-  if (variance_estimator %in% c("Stratified Multistage SRS", "Ultimate Cluster", "SD1", "SD2", "Deville-1", "Deville-2")) {
+  if (variance_estimator %in% c("Stratified Multistage SRS", "Ultimate Cluster", "SD1", "SD2", "Deville-1", "Deville-2", "Deville-Tille")) {
 
     use_sparse_matrix <- TRUE
 
     # Ensure the minimal set of inputs is supplied
-    if (variance_estimator %in% c("Stratified Multistage SRS", "Ultimate Cluster", "Deville-1", "Deville-2")) {
+    if (variance_estimator %in% c("Stratified Multistage SRS", "Ultimate Cluster", "Deville-1", "Deville-2", "Deville-Tille")) {
       if (is.null(cluster_ids) || is.null(strata_ids)) {
         sprintf(
           "For `variance_estimator='%s'`, must supply a matrix or data frame to both `strata_ids` and `cluster_ids`",
@@ -212,7 +219,7 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
         ) |> stop()
       }
     }
-    if (variance_estimator %in% c("Deville-1", "Deville-2")) {
+    if (variance_estimator %in% c("Deville-1", "Deville-2", "Deville-Tille")) {
       if (is.null(probs)) {
         sprintf(
           "For `variance_estimator='%s'`, must supply a matrix or data frame to `probs`.",
@@ -226,7 +233,7 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
         stop("For `variance_estimator='Stratified Multistage SRS'`, must supply a matrix or data frame to `strata_pop_sizes`.")
       }
     }
-    if (variance_estimator %in% c("SD1", "SD2", "Deville-1", "Deville-2")) {
+    if (variance_estimator %in% c("SD1", "SD2", "Deville-1", "Deville-2", "Deville-Tille")) {
       if (is.null(cluster_ids)) {
         sprintf(
           "For `variance_estimator='%s'`, must supply a matrix or data frame to `cluster_ids`",
@@ -242,6 +249,17 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
         ) |> stop()
       }
     }
+    if (variance_estimator == "Deville-Tille") {
+      if (missing(aux_vars) || is.null(aux_vars)) {
+        sprintf(
+          "For `variance_estimator='%s', must supply a matrix to `aux_vars`",
+          variance_estimator
+        ) |> stop()
+      }
+      if (!is.matrix(aux_vars)) {
+        stop("`aux_vars` must be a matrix.")
+      }
+    }
 
     # Determine number of stages and ultimate units,
     # and make clusters/strata explicit if they weren't supplied
@@ -263,6 +281,10 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
       number_of_ultimate_units <- nrow(strata_ids)
     }
 
+    if ((variance_estimator == "Deville-Tille") && (number_of_stages > 1)) {
+      message("For `variance_estimator = 'Deville-Tille', the quadratic form only takes into account the first stage of sampling.")
+    }
+
     # Make sure each stage's sampling units are nested within strata
     # and each stage's sampling units are nested
     # within previous stage sampling units
@@ -288,8 +310,8 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
 
     if (is.null(strata_pop_sizes)) {
       strata_pop_sizes <- Matrix::Matrix(data = Inf,
-                                 nrow = number_of_ultimate_units,
-                                 ncol = number_of_stages)
+                                         nrow = number_of_ultimate_units,
+                                         ncol = number_of_stages)
     }
   }
 
@@ -319,8 +341,8 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
 
   if (variance_estimator == "Ultimate Cluster") {
     quad_form_matrix <- Matrix::Matrix(data = 0,
-                               nrow = number_of_ultimate_units,
-                               ncol = number_of_ultimate_units) |>
+                                       nrow = number_of_ultimate_units,
+                                       ncol = number_of_ultimate_units) |>
       as("symmetricMatrix")
     if (use_sparse_matrix) {
       quad_form_matrix <- quad_form_matrix |> as("CsparseMatrix")
@@ -340,7 +362,7 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
     }
   }
 
-  if (variance_estimator %in% c("Stratified Multistage SRS", "Deville-1", "Deville-2")) {
+  if (variance_estimator %in% c("Stratified Multistage SRS", "Deville-1", "Deville-2", "Deville-Tille")) {
     quad_form_matrix <- Matrix::Matrix(data = 0,
                                        nrow = number_of_ultimate_units,
                                        ncol = number_of_ultimate_units) |>
@@ -401,7 +423,7 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
               probs = cluster_probs,
               method = variance_estimator
             ),
-            cluster_ids = cluster_ids[stratum_indices, stage, drop = TRUE],
+            cluster_ids = current_cluster_ids,
             rows = TRUE, cols = TRUE
           )
           # Get product of sampling probabilities from previous stages
@@ -414,6 +436,26 @@ make_quad_form_matrix <- function(variance_estimator = "Yates-Grundy",
 
         }
 
+        if ((variance_estimator == "Deville-Tille") && (stage == 1)) {
+          # Get quadratic form at current stage
+          current_cluster_ids <- cluster_ids[stratum_indices, stage, drop = TRUE]
+          current_probs <- probs[stratum_indices, stage, drop = TRUE]
+          cluster_probs <- current_probs[!duplicated(current_cluster_ids)]
+          current_aux_vars <- aux_vars[stratum_indices, , drop = FALSE]
+          cluster_aux_vars <- current_aux_vars[!duplicated(current_cluster_ids), , drop = FALSE]
+
+          Q_current <- distribute_matrix_across_clusters(
+            cluster_level_matrix = make_deville_tille_matrix(
+              probs = cluster_probs,
+              aux_vars = cluster_aux_vars
+            ),
+            cluster_ids = current_cluster_ids,
+            rows = TRUE, cols = TRUE
+          )
+
+          prev_stages_samp_prob <- 1
+        }
+
         # Add overall variance contribution from current stage/stratum sampling
         quad_form_matrix[stratum_indices,stratum_indices] <- (
           quad_form_matrix[stratum_indices,stratum_indices] +
@@ -677,6 +719,92 @@ make_ppswor_approx_matrix <- function(probs, method = "Deville-1") {
   return(Sigma)
 }
 
+#' @title Create a quadratic form's matrix
+#' for a Deville-Tillé variance estimator for balanced samples
+#' @description Creates the quadratic form matrix for a
+#' variance estimator for balanced samples,
+#' proposed by Deville and Tillé (2005).
+#' @param probs A vector of first-order inclusion probabilities
+#' @param aux_vars A matrix of auxiliary variables,
+#' with the number of rows matching the number of elements of \code{probs}.
+#'
+#' @return A symmetric matrix whose dimension matches the length of \code{probs}.
+#' @keywords internal
+#' @details
+#' See Section 6.8 of Tillé (2020) for more detail on this estimator,
+#' including an explanation of its quadratic form.
+#' See Deville and Tillé (2005) for the results of a simulation study
+#' comparing this and other alternative estimators for balanced sampling.
+#'
+#' The estimator can be written as follows:
+#' \deqn{
+#'   v(\hat{Y})=\sum_{k \in S} \frac{c_k}{\pi_k^2}\left(y_k-\hat{y}_k^*\right)^2,
+#' }
+#' where
+#' \deqn{
+#'   \hat{y}_k^*=\mathbf{z}_k^{\top}\left(\sum_{\ell \in S} c_{\ell} \frac{\mathbf{z}_{\ell} \mathbf{z}_{\ell}^{\prime}}{\pi_{\ell}^2}\right)^{-1} \sum_{\ell \in S} c_{\ell} \frac{\mathbf{z}_{\ell} y_{\ell}}{\pi_{\ell}^2}
+#' }
+#' and \eqn{\mathbf{z}_k} denotes the vector of auxiliary variables for observation \eqn{k}
+#' included in sample \eqn{S}, with inclusion probability \eqn{\pi_k}. The value \eqn{c_k} is set to \eqn{\frac{n}{n-q}(1-\pi_k)},
+#' where \eqn{n} is the number of observations and \eqn{q} is the number of auxiliary variables.
+#'
+#' See Li, Chen, and Krenzke (2014) for an example of this estimator's use
+#' as the basis for a generalized replication estimator. See Breidt and Chauvet (2011)
+#' for a discussion of alternative simulation-based estimators for the specific
+#' application of variance estimation for balanced samples selected using the cube method.
+#'
+#' @references
+#' - Breidt, F.J. and Chauvet, G. (2011).
+#' "Improved variance estimation for balanced samples drawn via the cube method."
+#' Journal of Statistical Planning and Inference, 141, 411-425.
+#'
+#' - Deville, J.‐C., and Tillé, Y. (2005). "\emph{Variance approximation under balanced sampling.}"
+#' \strong{Journal of Statistical Planning and Inference}, 128, 569–591.
+#'
+#' - Li, J., Chen, S., and Krenzke, T. (2014).
+#' "Replication Variance Estimation for Balanced Sampling: An Application to the PIAAC Study."
+#' Proceedings of the Survey Research Methods Section, 2014: 985–994. Alexandria, VA: American Statistical Association.
+#' http://www.asasrms.org/Proceedings/papers/1984_094.pdf.
+#'
+#' - Tillé, Y. (2020). "\emph{Sampling and estimation from finite populations}." (I. Hekimi, Trans.). Wiley.
+make_deville_tille_matrix <- function(probs, aux_vars) {
+
+  if (all(probs == 1)) {
+    return(matrix(0, length(probs), length(probs)))
+  }
+
+  n <- length(probs)
+
+  q <- ncol(aux_vars)
+
+  if (is.null(q) || (q < 1)) {
+    stop("`aux_vars` must be a matrix with at least one column")
+  }
+  if (q >= n) {
+    error_msg <- paste(
+      "The number of columns of `aux_vars` exceeds the number of observations:",
+      "the estimator is undefined in this case."
+    )
+    stop(error_msg)
+  }
+
+  # Calculate the constants used for each observation
+  c_k <- (1 - probs) * (n/(n-q))
+
+  # Calculate the hat matrix for creating y-star
+  H <- wls_hat_matrix(X = aux_vars, w = c_k/(probs^2))
+
+  # Subtract the hat matrix from the identity matrix
+  I_minus_H <- diag(n) - H
+
+  # Weight each entry
+  wtd_I_minus_H <- I_minus_H * sqrt(c_k/(probs^2))
+
+  Sigma <- diag(probs) %*% crossprod(wtd_I_minus_H, wtd_I_minus_H) %*% diag(probs)
+
+  return(Sigma)
+}
+
 #' @title Helper function to turn a cluster-level matrix into an element-level matrix
 #' by duplicating rows or columns of the matrix
 #' @description Turns a cluster-level matrix into an element-level matrix
@@ -840,6 +968,41 @@ ht_matrix_to_joint_probs <- function(ht_quad_form) {
   return(joint_probs)
 }
 
+#' @title Create the "hat matrix" for weighted least squares regression
+#'
+#' @param X Matrix of predictor variables, with \code{n} rows
+#' @param w Vector of weights (should all be nonnegative), of length \code{n}
+#'
+#' @return An \eqn{n \times n} matrix. This is the "hat matrix" for a WLS regression.
+#' @keywords internal
+#'
+wls_hat_matrix <- function(X, w) {
+
+  # Temporarily drop entries with zero weights
+  n <- length(w)
+  which_zero <- (w == 0)
+  X <- X[(w != 0), , drop = FALSE]
+  w <- w[(w != 0)]
+
+  # Compute the hat matrix
+  rw <- sqrt(w)
+  X1 <- rw * X
+
+  QR <- qr.default(X1, LAPACK = TRUE)
+  Q <- qr.qy(QR, diag(1, nrow = nrow(QR$qr), ncol = QR$rank))
+
+  Q1 <- (1 / rw) * Q
+  Q2 <- rw * Q
+
+  H <- tcrossprod(Q1, Q2)
+
+  # Restore entries with zero weights
+  result <- matrix(0, nrow = n, ncol = n)
+  result[!which_zero, !which_zero] <- H
+
+  return(result)
+}
+
 #' @title Combine quadratic forms from each phase of a two phase design
 #' @description This function combines quadratic forms from each phase of a two phase design,
 #' so that the combined variance of the entire two-phase sampling design can be estimated.
